@@ -13,6 +13,7 @@
 
 #include "logtofile_bgw.h"
 #include "logtofile_connect.h"
+#include "logtofile_execution_hook.h"
 #include "logtofile_guc.h"
 #include "logtofile_log.h"
 #include "logtofile_shmem.h"
@@ -116,6 +117,22 @@ void _PG_init(void)
       PGC_SIGHUP, GUC_NOT_IN_SAMPLE | GUC_SUPERUSER_ONLY,
       PgAuditLogToFile_guc_check_log_format, NULL, NULL);
 
+  DefineCustomBoolVariable(
+      "pgaudit.log_execution_time",
+      "Logs the execution time of each statement.", NULL,
+      &guc_pgaudit_ltf_log_execution_time,
+      false,
+      PGC_POSTMASTER, GUC_NOT_IN_SAMPLE | GUC_SUPERUSER_ONLY,
+      NULL, NULL, NULL);
+
+  DefineCustomBoolVariable(
+      "pgaudit.log_execution_memory",
+      "Logs the memory usage of each statement.", NULL,
+      &guc_pgaudit_ltf_log_execution_memory,
+      false,
+      PGC_POSTMASTER, GUC_NOT_IN_SAMPLE | GUC_SUPERUSER_ONLY,
+      NULL, NULL, NULL);
+
   EmitWarningsOnPlaceholders("pgauditlogtofile");
 
   /* background worker */
@@ -131,6 +148,17 @@ void _PG_init(void)
 
   RegisterBackgroundWorker(&worker);
 
+  /* Executor hooks for execution measurement */
+  if (guc_pgaudit_ltf_log_execution_time || guc_pgaudit_ltf_log_execution_memory)
+  {
+    pgaudit_ltf_prev_ExecutorStart = ExecutorStart_hook;
+    ExecutorStart_hook = PgAuditLogToFile_ExecutorStart_Hook;
+    pgaudit_ltf_prev_ExecutorEnd = ExecutorEnd_hook;
+    ExecutorEnd_hook = PgAuditLogToFile_ExecutorEnd_Hook;
+    pgaudit_ltf_prev_ExecutorRun = ExecutorRun_hook;
+    ExecutorRun_hook = PgAuditLogToFile_ExecutorRun_Hook;
+  }
+
 /* backend hooks */
 #if (PG_VERSION_NUM >= 150000)
   pgaudit_ltf_prev_shmem_request_hook = shmem_request_hook;
@@ -155,4 +183,11 @@ void _PG_fini(void)
 {
   emit_log_hook = pgaudit_ltf_prev_emit_log_hook;
   shmem_startup_hook = pgaudit_ltf_prev_shmem_startup_hook;
+
+  /* Executor hooks for execution measurement */
+  if (guc_pgaudit_ltf_log_execution_time || guc_pgaudit_ltf_log_execution_memory)
+  {
+    ExecutorStart_hook = pgaudit_ltf_prev_ExecutorStart;
+    ExecutorEnd_hook = pgaudit_ltf_prev_ExecutorEnd;
+  }
 }
