@@ -12,6 +12,7 @@
 #include "logtofile_csv.h"
 
 #include "logtofile_string_format.h"
+#include "logtofile_vars.h"
 
 #include <access/xact.h>
 #include <miscadmin.h>
@@ -20,6 +21,7 @@
 #include <tcop/tcopprot.h>
 #include <utils/json.h>
 #include <utils/ps_status.h>
+#include <utils/timestamp.h>
 
 #include <stdarg.h>
 
@@ -42,13 +44,13 @@ inline static void pgauditlogtofile_pgaudit_escape(StringInfo buf, char *line)
 void PgAuditLogToFile_csv_audit(StringInfo buf, const ErrorData *edata, int exclude_nchars)
 {
   bool print_stmt = false;
-  char *formatted_log_time;
+  char formatted_log_time[FORMATTED_TS_LEN];
+  instr_time now_instr;
 
-  /* timestamp with milliseconds */
-
-  formatted_log_time = PgAuditLogToFile_format_now_timestamp_millis();
+  /* timestamp with nanoseconds */
+  INSTR_TIME_SET_CURRENT(now_instr);
+  PgAuditLogToFile_format_instr_time_nanos(now_instr, formatted_log_time, sizeof(formatted_log_time));
   pgauditlogtofile_append_csv_value(buf, formatted_log_time);
-  pfree(formatted_log_time);
   appendStringInfoCharMacro(buf, ',');
 
   /* username */
@@ -172,6 +174,49 @@ void PgAuditLogToFile_csv_audit(StringInfo buf, const ErrorData *edata, int excl
   /* application name */
   if (application_name)
     pgauditlogtofile_append_csv_value(buf, application_name);
+  appendStringInfoCharMacro(buf, ',');
+
+  /* execution time */
+  if (guc_pgaudit_ltf_log_execution_time)
+  {
+    double total_time;
+    instr_time duration = pgaudit_ltf_statement_end_time;
+
+    /* start time */
+    PgAuditLogToFile_format_instr_time_nanos(pgaudit_ltf_statement_start_time, formatted_log_time, sizeof(formatted_log_time));
+    pgauditlogtofile_append_csv_value(buf, formatted_log_time);
+    appendStringInfoCharMacro(buf, ',');
+    
+    /* end time */
+    PgAuditLogToFile_format_instr_time_nanos(pgaudit_ltf_statement_end_time, formatted_log_time, sizeof(formatted_log_time));
+    pgauditlogtofile_append_csv_value(buf, formatted_log_time);
+    appendStringInfoCharMacro(buf, ',');
+
+    /* execution time */
+    INSTR_TIME_SUBTRACT(duration, pgaudit_ltf_statement_start_time);
+    total_time = INSTR_TIME_GET_DOUBLE(duration);
+    pgauditlogtofile_append_csv_fmt(buf, "%.9f", total_time);
+  }
+  appendStringInfoCharMacro(buf, ',');
+
+  /* memory usage */
+  if (guc_pgaudit_ltf_log_execution_memory)
+  {
+    Size memory_usage = pgaudit_ltf_statement_memory_end - pgaudit_ltf_statement_memory_start;
+    pgauditlogtofile_append_csv_fmt(buf, "%ld", pgaudit_ltf_statement_memory_start);
+    appendStringInfoCharMacro(buf, ',');
+    pgauditlogtofile_append_csv_fmt(buf, "%ld", pgaudit_ltf_statement_memory_end);
+    appendStringInfoCharMacro(buf, ',');
+    pgauditlogtofile_append_csv_fmt(buf, "%ld", pgaudit_ltf_statement_memory_peak);
+    appendStringInfoCharMacro(buf, ',');
+    pgauditlogtofile_append_csv_fmt(buf, "%ld", memory_usage < 0 ? 0 : memory_usage);
+  }
+  else
+  {
+    appendStringInfoCharMacro(buf, ',');
+    appendStringInfoCharMacro(buf, ',');
+    appendStringInfoCharMacro(buf, ',');
+  }
 
   appendStringInfoCharMacro(buf, '\n');
 }
