@@ -33,7 +33,6 @@
 /* Defines */
 #define PGAUDIT_PREFIX_LINE "AUDIT: "
 #define PGAUDIT_PREFIX_LINE_LENGTH sizeof(PGAUDIT_PREFIX_LINE) - 1
-#define FORMATTED_TS_LEN 128
 
 /*
  * We really want line-buffered mode for logfile output, but Windows does
@@ -49,6 +48,7 @@
 /* variables to use only in this unit */
 static char filename_in_use[MAXPGPATH];
 static int autoclose_thread_status_debug = 0; // 0: new proc, 1: th running, 2: th running sleep used, 3: th closed
+static uint32 pgaudit_ltf_local_rotation_generation = 0;
 
 /* forward declaration private functions */
 void pgauditlogtofile_close_file(void);
@@ -126,16 +126,28 @@ bool pgauditlogtofile_is_enabled(void)
 bool pgauditlogtofile_record_audit(const ErrorData *edata, int exclude_nchars)
 {
   bool rc;
+  char shm_filename[MAXPGPATH];
+  uint32 current_generation;
 
-  ereport(DEBUG5, (errmsg("pgauditlogtofile record audit in %s (shm %s)",
-                          filename_in_use, pgaudit_ltf_shm->filename)));
-  /* do we need to rotate? */
-  if (strlen(pgaudit_ltf_shm->filename) > 0 && strcmp(filename_in_use, pgaudit_ltf_shm->filename) != 0)
+  current_generation = pg_atomic_read_u32(&pgaudit_ltf_shm->rotation_generation);
+  if (current_generation != pgaudit_ltf_local_rotation_generation || strlen(filename_in_use) == 0)
   {
-    ereport(DEBUG3, (
-                        errmsg("pgauditlogtofile record audit file handler requires reopening - shm_filename %s filename_in_use %s",
-                               pgaudit_ltf_shm->filename, filename_in_use)));
-    pgauditlogtofile_close_file();
+    LWLockAcquire(pgaudit_ltf_shm->lock, LW_SHARED);
+    strlcpy(shm_filename, pgaudit_ltf_shm->filename, MAXPGPATH);
+    LWLockRelease(pgaudit_ltf_shm->lock);
+
+    pgaudit_ltf_local_rotation_generation = current_generation;
+
+    ereport(DEBUG5, (errmsg("pgauditlogtofile record audit in %s (shm %s)",
+                            filename_in_use, shm_filename)));
+    /* do we need to rotate? */
+    if (strlen(shm_filename) > 0 && strcmp(filename_in_use, shm_filename) != 0)
+    {
+      ereport(DEBUG3, (
+                          errmsg("pgauditlogtofile record audit file handler requires reopening - shm_filename %s filename_in_use %s",
+                                 shm_filename, filename_in_use)));
+      pgauditlogtofile_close_file();
+    }
   }
 
   if (!pgauditlogtofile_is_open_file() && !pgauditlogtofile_open_file())
@@ -225,9 +237,14 @@ bool pgauditlogtofile_open_file(void)
 {
   mode_t oumask;
   bool opened = false;
+  char shm_filename[MAXPGPATH];
+
+  LWLockAcquire(pgaudit_ltf_shm->lock, LW_SHARED);
+  strlcpy(shm_filename, pgaudit_ltf_shm->filename, MAXPGPATH);
+  LWLockRelease(pgaudit_ltf_shm->lock);
 
   // if the filename is empty, we short-circuit
-  if (strlen(pgaudit_ltf_shm->filename) == 0)
+  if (strlen(shm_filename) == 0)
     return opened;
 
   /* Create spool directory if not present; ignore errors */
@@ -239,7 +256,7 @@ bool pgauditlogtofile_open_file(void)
    */
   oumask = umask(
       (mode_t)((~(guc_pgaudit_ltf_log_file_mode | S_IWUSR)) & (S_IRWXU | S_IRWXG | S_IRWXO)));
-  pgaudit_ltf_file_handler = fopen(pgaudit_ltf_shm->filename, "a");
+  pgaudit_ltf_file_handler = fopen(shm_filename, "a");
   umask(oumask);
 
   if (pgaudit_ltf_file_handler)
@@ -252,13 +269,13 @@ bool pgauditlogtofile_open_file(void)
     _setmode(_fileno(file_handler), _O_TEXT);
 #endif
     // File open, we update the filename we are using
-    strcpy(filename_in_use, pgaudit_ltf_shm->filename);
+    strcpy(filename_in_use, shm_filename);
   }
   else
   {
     ereport(LOG_SERVER_ONLY,
             (errcode_for_file_access(),
-             errmsg("could not open log file \"%s\": %m", pgaudit_ltf_shm->filename)));
+             errmsg("could not open log file \"%s\": %m", shm_filename)));
   }
 
   return opened;
@@ -310,6 +327,7 @@ bool pgauditlogtofile_write_audit(const ErrorData *edata, int exclude_nchars)
       pgauditlogtofile_close_file();
     }
   }
+  pfree(buf.data);
 
   return success;
 }
\ No newline at end of file
