@@ -138,6 +138,7 @@ void PgAuditLogToFile_shmem_startup(void)
     pfree(prefixes);
 
     pgaudit_ltf_shm->lock = &(GetNamedLWLockTranche("pgauditlogtofile"))->lock;
+    pg_atomic_init_u32(&pgaudit_ltf_shm->rotation_generation, 0);
     PgAuditLogToFile_calculate_current_filename();
     PgAuditLogToFile_set_next_rotation_time();
   }
@@ -191,6 +192,10 @@ void PgAuditLogToFile_calculate_current_filename(void)
   LWLockAcquire(pgaudit_ltf_shm->lock, LW_EXCLUSIVE);
   memset(pgaudit_ltf_shm->filename, 0, sizeof(pgaudit_ltf_shm->filename));
   strcpy(pgaudit_ltf_shm->filename, filename);
+  if (pg_atomic_read_u32(&pgaudit_ltf_shm->rotation_generation) == PG_UINT32_MAX)
+    pg_atomic_write_u32(&pgaudit_ltf_shm->rotation_generation, 0);
+  else
+    pg_atomic_add_fetch_u32(&pgaudit_ltf_shm->rotation_generation, 1);
   LWLockRelease(pgaudit_ltf_shm->lock);
 
   pfree(filename);
