@@ -14,11 +14,14 @@
 
 #include <postgres.h>
 #include <datatype/timestamp.h>
+#include <executor/executor.h>
 #include <miscadmin.h>
 #include <pgtime.h>
 #include <port/atomics.h>
+#include <portability/instr_time.h>
 #include <storage/ipc.h>
 #include <storage/lwlock.h>
+#include <utils/timestamp.h>
 
 #include <pthread.h>
 
@@ -31,6 +34,8 @@ extern bool guc_pgaudit_ltf_log_connections;
 extern bool guc_pgaudit_ltf_log_disconnections;
 extern int guc_pgaudit_ltf_auto_close_minutes;
 extern char *guc_pgaudit_ltf_log_format;
+extern bool guc_pgaudit_ltf_log_execution_time;
+extern bool guc_pgaudit_ltf_log_execution_memory;
 
 // Audit log file handler
 extern FILE *pgaudit_ltf_file_handler;
@@ -41,9 +46,23 @@ extern pthread_t pgaudit_ltf_autoclose_thread;
 extern pthread_attr_t pgaudit_ltf_autoclose_thread_attr;
 extern Timestamp pgaudit_ltf_autoclose_active_ts;
 
+// Statement time measurement
+extern instr_time pgaudit_ltf_statement_start_time;
+extern instr_time pgaudit_ltf_statement_end_time;
+
+// Statement memory measurement
+extern Size pgaudit_ltf_statement_memory_start;
+extern Size pgaudit_ltf_statement_memory_end;
+extern Size pgaudit_ltf_statement_memory_peak;
+
 // Hook log
 extern emit_log_hook_type pgaudit_ltf_prev_emit_log_hook;
 
+// Executor Hook
+extern ExecutorStart_hook_type pgaudit_ltf_prev_ExecutorStart;
+extern ExecutorRun_hook_type pgaudit_ltf_prev_ExecutorRun;
+extern ExecutorEnd_hook_type pgaudit_ltf_prev_ExecutorEnd;
+
 // Shared Memory types
 typedef struct PgAuditLogToFilePrefix
 {
@@ -60,6 +79,7 @@ typedef struct pgAuditLogToFileShm
   size_t num_prefixes_disconnection;
   char filename[MAXPGPATH];
   pg_time_t next_rotation_time;
+  pg_atomic_uint32 rotation_generation;
 } PgAuditLogToFileShm;
 
 // Shared Memory
